#include "Blackboard.H"
#include "TotalPF.H"
#include "TBConfig.H"
#include "UnitAgent.H"

using namespace std;

const sint4 Blackboard::TYPE_TANK = 0;
const sint4 Blackboard::TYPE_MARINE = 1;
const sint4 Blackboard::TYPE_BASE = 2;

sint4 Blackboard::state = 1;
bool Blackboard::state_locked = false;
const sint4 Blackboard::STATE_HUNT_UNITS = 1;
const sint4 Blackboard::STATE_HUNT_BASES = 2;

/*
	Switch state for the overall tactics.
*/
void Blackboard::setState(sint4 mState) {
	state = mState;
}

/*
	Locks the state to the specified state
*/
void Blackboard::lockState(sint4 mState) {
	if (!state_locked) {
		state = mState;
		state_locked = true;
	}
}

/*
	Returns true if state is locked, false otherwise
*/
bool Blackboard::stateLocked() {
	return state_locked;
}

/*
	Updates the tactics state based on the current game state.
*/
void Blackboard::updateState(const Game& game) {
	if (state_locked) return;

	int n = game.get_player_num();
	const sint4 cid = game.get_client_player();
	
	int friendlyUnits = 0;
	int friendlyBases = 0;
	int enemyUnits = 0;
	int enemyBases = 0;

	//Iterate through all players.
	for (int player = 0; player < n; player++) {
		const Game::ObjCont &objs = game.get_objs(player);

		bool friendly = false;
		if (player == cid) {
			friendly = true;
		}

		//Iterate through all objects for this player
		FORALL (objs, it) {
			GameObjPtr gob = (*it)->get_GameObj();
			if (gob && gob->sod.in_game && *gob->sod.x >= 0 && !gob->is_dead()) {
				//Valid object found
				std::string objType = gob->bp_name();
				
				if (objType == "tank" || objType == "marine") {
					if (friendly) friendlyUnits++;
					else enemyUnits++;
				}
				if (objType == "controlCenter") {
					if (friendly) friendlyBases++;
					else enemyBases++;
				}
			}
		}
	}

	sint4 newState = Blackboard::STATE_HUNT_UNITS;

	if (enemyUnits <= 5) { //3
		newState = Blackboard::STATE_HUNT_BASES;
	}
	else {
		int unitRatio = friendlyUnits / enemyUnits;
		if (unitRatio >= 2) { //5
			newState = Blackboard::STATE_HUNT_BASES;
		}
	}

	if (friendlyBases == 1 && enemyUnits > 0) {
		newState = Blackboard::STATE_HUNT_UNITS;
	}
	if (enemyBases == 1) {
		newState = Blackboard::STATE_HUNT_BASES;
	}

	if (friendlyBases == 0) {
		//This is probably a game without bases
		newState = Blackboard::STATE_HUNT_UNITS;
	}
	
	if (newState != Blackboard::state) {
		Blackboard::state = newState;
	}

	//Blackboard::state = Blackboard::STATE_HUNT_BASES;

	//cout << "ou=" << friendlyUnits << " eu=" << enemyUnits << " ob=" << friendlyBases << " eb=" << enemyBases << endl;
}

/*
	Gets the type (tank, marine or base) for a unit.
*/
int Blackboard::getType(GameObjPtr unit) {
	std::string objType = unit->bp_name();

	if (objType == "tank") {
		return Blackboard::TYPE_TANK;
	}
	else if (objType == "marine") {
		return Blackboard::TYPE_MARINE;
	}
	else if (objType == "controlCenter") {
		return Blackboard::TYPE_BASE;
	}
	else {
		cout << "Unknown unit type:" << objType << endl;
		return Blackboard::TYPE_BASE;
	}
}
