#ifndef Blackboard_H
#define Blackboard_H

#include "GameObj.H"
#include "Game.H"
#include "TBConfig.H"

using namespace std;

/*
	This class represents a blackboard where the player units can communicate there actions to
	other player units. It also contains some methods for unit action coordination.
*/
class Blackboard {
	
	typedef GameObj* GameObjPtr;

	public:

		/*
			Gets the type (tank, marine or base) for a unit.
		*/
		static int getType(GameObjPtr unit);

		/*
			Switch state for the overall tactics.
		*/
		static void setState(sint4 mState);

		/*
			Locks the state to the specified state
		*/
		static void lockState(sint4 mState);

		/*
			Returns true if state is locked, false otherwise
		*/
		static bool stateLocked();

		/*
			Updates the tactics state based on the current game state.
		*/
		static void updateState(const Game& game);

		//Blackboard item for bases
		static const sint4 TYPE_BASE;
		//Blackboard item for tanks
		static const sint4 TYPE_TANK;
		//Blackboard item for marines
		static const sint4 TYPE_MARINE;

		//Default value to use if no value (location/id) exists
		static const sint4 NO_ID;

		//Current state for general tactics
		static sint4 state;
		//If set to true, state is not updated
		static bool state_locked;
		//Prioritize hunt enemy units
		static const sint4 STATE_HUNT_UNITS;
		//Prioritize hunt enemy bases
		static const sint4 STATE_HUNT_BASES;
};

#endif
