#ifndef ClientGUI_H
#define ClientGUI_H

#include "GameObj.H"
#include "TotalPF.H"
#include "PF.H"
#include "FireCoordinator.H"
#include <SDL/SDL_thread.h>

using namespace std;

/*
	Struct for handling RGB colors.
*/
struct RGB {
  uint1 R, G, B;
  uint4 col;

  void from(sint4 r, sint4 g, sint4 b=0) {
    R = r; G = g; B = b;
    col = ((((R << 8)+G) << 8) + B);
  }
  
  RGB(sint4 r=0, sint4 g=0, sint4 b=0) : R(r), G(g), B(b) {
    col = ((((R << 8)+G) << 8) + B);    
  }
  
  uint4 color() const { return col; }
  uint4 color_alpha(sint4 alpha=255) const {
    assert(alpha >=0 && alpha < 256);
    return (col << 8) + alpha;
  }
};

struct Attack {
	int unitId;
	int targetId;
	int cnt;

	bool isThisObj(int mUnitId) {
		if (unitId == mUnitId) {
			return true;
		}
		return false;
	}

	bool isActive() {
		if (cnt > 0) {
			return true;
		}
		return false;
	}

	void reset() {
		cnt = 10;
	}

	void fade() {
		cnt -= 2;
	}
};

/*
	This class is responsible for displaying a charge map GUI. It is used
	to debug the behaviour of the units.
*/
class ClientGUI {
	public:

		/*
			Constructor.
		*/
		ClientGUI();
		
		/*
			Updates the PF map.
		*/
		void setRef(TotalPF* map, PF* m_pf, GameStateModule* state, FireCoordinator* coord);

		/*
			Updates the GUI.
		*/
		void display();
		
	private:

		/*
			Returns a display color for the specified charge.
		*/
		Uint32 getColor(real8 charge);

		//GUI params
		SDL_Surface *screen;
		sint4 s_x, s_y;
		sint4 gfx_mag_frac;
		sint4 adj_x, adj_y;

		//PF map reference
		TotalPF* c_map;
		PF* pf;
		GameStateModule* gamestate;
		FireCoordinator* coordinator;

		vector<Attack> attacks;

		void updateAttack(int unitId, int targetId);

		int toScreenCoord(int coord);

		void drawAttackLine(int x, int y, int targetId, int fadeVal);
};

#endif
