#ifndef FCStructureItem_H
#define FCStructureItem_H

#include "GameObj.H"

using namespace std;

/*
	Structure for holding the units which can fire at the current target.
*/
struct AttackUnit {
	uint4 unitId;
	int unitType;
};

/*
	This class holds a target/unit combination in the fire coordination data structure.
	The structure contains one object of this class for each target one or more units can fire
	at. Each object has a list of the units which can fire at the target.
*/
class FCStructureItem {
	
	typedef GameObj* GameObjPtr;
	
	public:

		/*
			Adds an attacking unit for this target.
		*/
		void addAttacker(uint4 unitId, GameObjPtr unit);
		
		/*
			Returns true if there are enough attacking units to destroy
			this target.
		*/
		int canBeDestroyed();

		/*
			Removes a specified attacker unit from the list (if found).
		*/
		void removeAttacker(uint4 unitId);

		/*
			Returns true if this target is attacked by the specified attacker unit.
		*/
		bool attackedBy(uint4 unitId);

		/*
			Returns the number of attacking units for this target.
		*/
		uint4 getNoAttackers();

		/*
			Returns the object Id for the attacking unit at position [index] in the list.
		*/
		uint4 getAttackerId(uint4 index);

		//Operator overloading used for sorting the list
		bool operator==(const FCStructureItem &item) const;
		//Operator overloading used for sorting the list
      	bool operator<(const FCStructureItem &item) const;

		/*
			Dumps the data structure entry to the console window for debug purpose.
		*/
		void dump();

		//Object Id for this target.
		uint4 targetId;
		//GameObj for this target.
		GameObj* targetObj;
		//Current HP for this target.
		int HPleft;
		//Unit type for this target (i.e. tank or marine. See Blackboard).
		int targetUnitType;

		//List of attacking units for this target.
		vector<AttackUnit> units;

	private:
		
		/*
			Returns a value that is min + (max-min)*p%.
		*/
		int getValueBetween(int min, int max, real8 p);
};

#endif
