#include "PF.H"
#include "TotalPF.H"
#include "EventTimer.H"
#include "TBConfig.H"
#include "Blackboard.H"
#include "Options.H"

PF::PF() {
	sint4 seed;
  	Options::get("-seed", seed);
	
	rand.srand(seed);
	diffScoreSum = 0;
}

/*
	Bot Versions:
	1 - Static, medium difficulty
	2 - Static, low difficulty
	3 - Adaptive
	4 - Adaptive, let player win by dropping prob to min when player units <= 5
	5 - Adaptive, high learningrate
*/
real8 PF::getProbability(uint4 objId) {
	//Dynamic
#if DYNAMIC_AI == 1
	vector<ObjFuturePos>::iterator ou = units.begin();
	while (ou != units.end()) {
		if (ou->isThisObj(objId)) {
			return ou->prob;
		}
		ou++;
	}
	return 0;
#else

	//Static v1
	//return 0.3;
	//Static v2
	//return 0.1;

	//Full
	return 1;
#endif
}

void PF::updateProbabilities() {
	//Dynamic
#if DYNAMIC_AI == 1
	//Calculate current game score
	real8 aimScore = 0;
	real8 isScore = 0;
	int noEu = 0;

	//Enemy units
	vector<ObjPos>::iterator eu = enemies.begin();
	while (eu != enemies.end()) {
		isScore = isScore - 1 - eu->getHealthPct() * 0.5;
		eu++;
		noEu++;
	}

	//Own units
	vector<ObjFuturePos>::iterator ou = units.begin();
	while (ou != units.end()) {
		isScore = isScore + 1 + ou->getHealthPct() * 0.5;
		ou++;
	}

	//Own bases
	vector<BasePos>::iterator ob = bases.begin();
	while (ob != bases.end()) {
		//isScore += 2;
		ob++;
	}

	//Enemy bases
	vector<BasePos>::iterator eb = enemyBases.begin();
	while (eb != enemyBases.end()) {
		//isScore -= 2;
		eb++;
	}

	//Diff in score
	//p-regulator: Just consider current state
	real8 diffScore = (isScore - aimScore) * (real8)-1;
	//pd-regulator: Consider current and past states
	diffScoreSum += diffScore;
	//diffScore = diffScoreSum; //Uncomment to use pd-regulator instead of p-regulator
	//If all opponent units are down, end the game as quickly as possible
	if (noEu == 0) {
		diffScore = 100;
	}
	

	//Update probabilities
	real8 learnRate = 0.01; //0.01 for v3-4, 0.2 for v5.
	vector<ObjFuturePos>::iterator it = units.begin();
	while (it != units.end()) {
		
		real8 change = learnRate * diffScore;

		it->prob = it->prob + change;

		//Min/max check
		if (it->prob > 1) {
			it->prob = 1;
		}
		if (it->prob < 0.05) {
			it->prob = 0.05;
		}

		//4 - Adaptive, let player win by dropping prob to min when player units <= 5
		/*if (noEu <= 5 && noEu > 0) {
			it->prob = 0.02;
		}*/
		//cout << "[" << it->objId << "] -> prob=" << it->prob << endl;

		it++;
	}
	
	//cout << "score=" << (isScore - aimScore) << endl;
#else
	return;
#endif
}

int PF::getOU() {
	return units.size();
}

int PF::getEU() {
	return enemies.size();
}

int PF::getOB() {
	return bases.size();
}

int PF::getEB() {
	return enemyBases.size();
}

vector<ObjFuturePos>* PF::getUnitsList() {
	return &units;
}

vector<ObjPos>* PF::getEnemyUnitsList() {
	return &enemies;
}

vector<BasePos>* PF::getEnemyBasesList() {
	return &enemyBases;
}

/*
	Sets all own and enemy units as destroyed. Shall be called before updating the unit positions
	each frame. A unit which has its position updated are set as not destroyed, and after the
	position updates all units which still has the destroyed flag set can be removed.
*/
void PF::setAllDestroyed() {
	vector<ObjPos>::iterator it = enemies.begin();
	while (it != enemies.end()) {
		(*it).destroyed = true;
		(*it).visible = false;
		it++;
	}

	vector<ObjFuturePos>::iterator uit = units.begin();
	while (uit != units.end()) {
		(*uit).destroyed = true;
		uit++;
	}
}

void PF::checkVisibility() {
#if FULL_VISIBILITY == 1
	/*--- All enemy units visible ---*/
	vector<ObjPos>::iterator it = enemies.begin();
	while (it != enemies.end()) {
		(*it).visible = true;
		it++;
	}
#else
	/*--- Enemy units visible by one own unit = visible ---*/
	vector<ObjPos>::iterator it = enemies.begin();
	while (it != enemies.end()) {
		//Iterate through own units
		vector<ObjFuturePos>::iterator uit = units.begin();
		while (uit != units.end()) {
			real8 dist = TotalPF::distance(it->x, it->y, uit->x, uit->y);
			if (dist <= 120) {
				(*it).visible = true;
			}
			uit++;
		}

		it++;
	}
#endif
}

/*
	Removes destroyed own and enemy units.
	Must be used with the setAllDestroyed() method.
*/
void PF::cleanUp() {
	for (int i = 0; i < (int)enemies.size(); i++) {
		if (enemies[i].destroyed) {
			vector<ObjPos>::iterator it = enemies.begin();
			it += i;
			enemies.erase(it);
			i--;
		}
	}

	for (int i = 0; i < (int)units.size(); i++) {
		if (units[i].destroyed) {
			vector<ObjFuturePos>::iterator it = units.begin();
			it += i;
			units.erase(it);
			i--;
		}
	}
}

/*
	Updates the position for sheep [objId].
*/
void PF::updateSheepPosition(uint4 game_tick, uint4 objId, int x, int y) {
	//Check if struct for this unit exists and, if found, update
	bool found = false;
	vector<ObjPos>::iterator it = sheep.begin();
	while (it != sheep.end()) {
		if ((*it).isThisObj(objId)) {
			(*it).update(game_tick, x, y);
			found = true;
		}
		it++;
	}

	//Unit struct not found. Add new
	if (!found) {
		ObjPos pos;
		pos.objId = objId;
		pos.x = x;
		pos.y = y;
		pos.gametick = game_tick;
		sheep.push_back(pos);
	}
}

/*
	Returns the potential generated by sheep in the point (pX,pY).
*/
real8 PF::getSheepCharge(int pX, int pY, uint4 game_tick) {
	real8 p = 0;

	vector<ObjPos>::iterator it = sheep.begin();
	while (it != sheep.end()) {
		if (TotalPF::MHdistance(pX, pY, (*it).x, (*it).y) <= 26) {
			p += calcSheepCharge(pX, pY, (*it).x, (*it).y);
		}

		it++;
	}
	
	return p;
}

/*
	Returns the potential a sheep at (objX,objY) generates in the point (x,y).
*/
real8 PF::calcSheepCharge(int x, int y, int objX, int objY) {
	real8 p = 0;
	real8 dist = TotalPF::distance(x, y, objX, objY);

	if (bases.size() == 0) {
		//Game 4 -> Smaller sheep field
		if (dist <= 10) {
			if (dist <= 4) { //4  radius(sheep) = 4
				p = - 20;
			}
			else {
				p = -(20 - 2 * dist);
				if (p > 0) p = 0;
			}
		}
	}
	else {
		if (dist <= 8) {
			p = -20;
		}
		else {
			p = (real8)2 * dist - 26;
			if (p > 0) p = 0;
		}
		p = p * TBConfig::params.Sheep_w;
	}
	
	return p;
}

/*
	Updates the position for enemy unit [objId].
*/
void PF::updateEnemyUnitPosition(uint4 game_tick, uint4 objId, sint4 objType, int x, int y, int cHP) {
	//Check if struct for this unit exists and, if found, update
	bool found = false;
	vector<ObjPos>::iterator it = enemies.begin();
	while (it != enemies.end()) {
		if (it->isThisObj(objId)) {
			it->update(game_tick, x, y);
			it->HP = cHP;
			found = true;
			return;
		}
		it++;
	}

	//Unit struct not found. Add new
	if (!found) {
		ObjPos pos;
		pos.objId = objId;
		pos.type = objType;
		pos.x = x;
		pos.y = y;
		pos.gametick = game_tick;
		pos.HP = cHP;
		enemies.push_back(pos);
	}
}

/*
	Returns the potential generated by enemy units in the point (pX,pY).
*/
real8 PF::getEnemyUnitCharge(int pX, int pY, bool offensive, uint4 game_tick) {
	real8 p = 0;

	vector<ObjPos>::iterator it = enemies.begin();
	while (it != enemies.end()) {

		if (it->visible) { //TODO: Negate???
			if (!(*it).destroyed && (*it).isMarine()) {
				if (offensive) {
					real8 mp = calcOffensiveMarineCharge(pX, pY, (*it).x, (*it).y);
					//Use highest PF value
					if (p < mp) {
						p = mp;
					}
					//Sum fields
					//p += mp;
				}
				if (!offensive) {
					p += calcDefensiveMarineCharge(pX, pY, (*it).x, (*it).y);
				}
			}
	
			if (!(*it).destroyed && (*it).isTank()) {
				if (offensive) {
					real8 mp = calcOffensiveTankCharge(pX, pY, (*it).x, (*it).y);
					//Use highest PF value
					if (p < mp) {
						p = mp;
					}
					//Sum fields
					//p += mp;
				}
				if (!offensive) {
					p += calcDefensiveTankCharge(pX, pY, (*it).x, (*it).y);
				}
			}
		}

		it++;
	}
	
	return p;
}

/*
	Returns the potential a marine at (objX,objY) generates in the point (x,y).
*/
real8 PF::calcOffensiveMarineCharge(int x, int y, int objX, int objY) {
	real8 p = 0;
	real8 dist = TotalPF::distance(x, y, objX, objY);
	if (dist <= 63) {
		p = dist * 0.8 * 5;
	}
	else if (dist > 63 && dist <= 64) {
		p = 65 * 5;
	}
	else {
		p = (50 - (dist - 64) * 0.05) * 5;
		if (p < 0) p = 0;
	}

	return p;
}

/*
	Returns the potential a marine at (objX,objY) generates in the point (x,y).
*/
real8 PF::calcDefensiveMarineCharge(int x, int y, int objX, int objY) {
	if (TotalPF::MHdistance(x, y, objX, objY) > 144) return 0;

	real8 p = 0;
	real8 dist = TotalPF::distance(x, y, objX, objY);
	if (dist <= 72) { //72
		p = (-60 + (dist * 0.8)) * 10;
	}
	
	return p;
}

/*
	Returns the potential a tank at (objX,objY) generates in the point (x,y).
*/
real8 PF::calcOffensiveTankCharge(int x, int y, int objX, int objY) {
	real8 p = 0;
	real8 dist = TotalPF::distance(x, y, objX, objY);

	if (dist <= 110) {
		p = (real8)240 / (real8)110 * dist;
	}
	else if (dist > 110 && dist <= 112) {
		p = 240;
	}
	else {
		p = 240 - (dist - 112) * (real8)0.24;
		if (p < 0) p = 0;
	}
	p = p * TBConfig::params.OffTank_w;
	
	return p;
}

/*
	Returns the potential a tank at (objX,objY) generates in the point (x,y).
*/
real8 PF::calcDefensiveTankCharge(int x, int y, int objX, int objY) {
	real8 dist = TotalPF::distance(x, y, objX, objY);
	real8 p = -800 + dist * 6.4;
	if (p > 0) {
		p = 0;
	}
	p = p * TBConfig::params.DefTank_w;
	
	return p;
}

/*
	Updates the position for own unit [objId].
*/
void PF::updateUnitPosition(uint4 game_tick, uint4 objId, sint4 objType, int x, int y, int cHP) {
	//Check if struct for this unit exists and, if found, update
	bool found = false;
	vector<ObjFuturePos>::iterator it = units.begin();
	while (it != units.end()) {
		if (it->isThisObj(objId)) {
			it->update(game_tick, x, y);
			it->HP = cHP;
			found = true;

			//Update trail
			(*it).trail.update(x, y, game_tick);
			
			return;
		}
		it++;
	}

	//Unit struct not found. Add new
	if (!found) {
		ObjFuturePos pos;
		pos.objId = objId;
		pos.type = objType;
		pos.x = x;
		pos.y = y;
		pos.gametick = game_tick;
		pos.prob = 0.4;
		pos.HP = cHP;
		units.push_back(pos);

		//Init trail
		Trail trail;
		trail.init(x, y, game_tick);
		pos.trail = trail;
	}
}

/*
	Sets the position the unit [objId] is moving to.
*/
void PF::setNextUnitPosition(uint4 objId, int nextX, int nextY) {
	vector<ObjFuturePos>::iterator it = units.begin();
	while (it != units.end()) {
		if ((*it).isThisObj(objId)) {
			(*it).nextX = nextX;
			(*it).nextY = nextY;
			return;
		}
		it++;
	}
}

/*
	Returns the potential generated by own units in the point (pX,pY).
*/
real8 PF::getUnitCharge(int pX, int pY, uint4 objId, uint4 game_tick) {
	real8 p = 0;

	vector<ObjFuturePos>::iterator it = units.begin();
	while (it != units.end()) {
		if (!(*it).destroyed && (*it).isMarine() && (*it).objId != objId) {
			p += calcMarineUnitCharge(pX, pY, (*it).x, (*it).y);
			if ((*it).hasNextPos()) {
				p += calcMarineUnitCharge(pX, pY, (*it).nextX, (*it).nextY);
			}
		}
		if (!(*it).destroyed && (*it).isTank() && (*it).objId != objId) {
			p += calcTankUnitCharge(pX, pY, (*it).x, (*it).y);
			if ((*it).hasNextPos()) {
				p += calcTankUnitCharge(pX, pY, (*it).nextX, (*it).nextY);
			}
		}

		//Trail charge
		if ((*it).isThisObj(objId)) {
			real8 v = it->trail.getP(pX, pY, game_tick);
			p += it->trail.getP(pX, pY, game_tick);
		}

		it++;
	}
	
	return p;
}

/*
	Returns the potential an own marine at (objX,objY) generates in the point (x,y).
*/
real8 PF::calcMarineUnitCharge(int x, int y, int objX, int objY) {
	if (TotalPF::MHdistance(x, y, objX, objY) > 20) return 0;

	real8 p = 0;
	real8 dist = TotalPF::distance(x, y, objX, objY);
	if (dist <= 8) { //radius(marine) = 4
		p = p - 20;
	}
	else {
		real8 mp = 20 - 2 * dist; //20 - 2 * dist
		if (mp < 0) mp = 0;

		p = p - mp;
	}
	
	return p;
}

/*
	Returns the potential generated by the marine coordination field in the point (pX,pY).
*/
real8 PF::getMarineCoordCharge(int pX, int pY, uint4 game_tick, int map_width) {
	real8 p = 0;

	int diffX = pX - 1;
	bool startAtLeft = false;
	if (pX <= map_width / 2) {
		startAtLeft = true;
	}
	
	if (!startAtLeft) {
		diffX = map_width - pX - 1;
	}
	
	if (diffX <= 224) {
		p += (real8)diffX / (real8)8 * (real8)3;
	}
	else if (diffX > 224 && diffX <= 248) {
		p += 92;
	}
	else if (diffX > 248 && diffX < 272) {
		p += 92 - (diffX - 248) * 0.3;
	}
	
	return p;
}

/*
	Returns the potential an own tank at (objX,objY) generates in the point (x,y).
*/
real8 PF::calcTankUnitCharge(int x, int y, int objX, int objY) {
	real8 p = 0;
	real8 dist = TotalPF::distance(x, y, objX, objY);

	if (dist <= 14) {
		p = p - 20;
	}
	else  {
		real8 mp = 32 - dist * 2;
		if (mp < 0) mp = 0;
		p = p - mp;
	}
	p = p * TBConfig::params.Tank_w;
	
	return p;
}

/*
	Removes all elements in the own- and enemy bases vectors.
*/
void PF::clearBaseLists() {
	bases.clear();
	enemyBases.clear();
}

/*
	Adds the base [objId] to a bases vector.
*/
void PF::addBase(uint4 objId, int x, int y, int x1, int y1, int x2, int y2, bool own) {
	BasePos pos;
	pos.objId = objId;
	pos.x = x;
	pos.y = y;
	pos.x1 = x1;
	pos.y1 = y1;
	pos.x2 = x2;
	pos.y2 = y2;
	if (own) {
		bases.push_back(pos);
	}
	else {
		enemyBases.push_back(pos);
	}
}

/*
	Returns the potential generated by own bases in the point (pX,pY).
*/
real8 PF::getBaseCharge(int pX, int pY) {
	real8 p = 0;

	vector<BasePos>::iterator it = bases.begin();
	while (it != bases.end()) {
		p += calcBaseCharge(pX, pY, (*it).x, (*it).y, (*it).isPositionInside(pX, pY));
		it++;
	}

	return p;
}

/*
	Returns the potential an own base at (objX,objY) generates in the point (x,y).
*/
real8 PF::calcBaseCharge(int x, int y, int objX, int objY, bool inside) {
	real8 p = 0;
	if (inside) {
		p = TotalPF::CHARGE_IMPASSABLE_TILES * 10;
	}
	else {
		real8 dist = TotalPF::distance(x, y, objX, objY);

		if (dist <= 43) {
			//Base width = 62
			p = (real8)(6 * dist - 258);
		}
		else {
			real8 useDist = dist - 43;
			p = 50 - dist * 0.02;
			if (p < 0) {
				p = 0;
			}
		}
	}
	p = p * TBConfig::params.Base_w;

	return p;
}

/*
	Returns the potential generated by enemy bases in the point (pX,pY).
*/
real8 PF::getEnemyBaseCharge(int pX, int pY) {
	real8 p = 0;

	vector<BasePos>::iterator it = enemyBases.begin();
	while (it != enemyBases.end()) {
		if (Blackboard::state == Blackboard::STATE_HUNT_UNITS) {
			p += calcBaseCharge(pX, pY, (*it).x, (*it).y, (*it).isPositionInside(pX, pY));
		}
		else {
			real8 mp = calcEnemyBaseCharge(pX, pY, (*it).x, (*it).y, (*it).isPositionInside(pX, pY));
			//Use highest potential
			if (mp > p) {
				p = mp;
			}
			//Sum potentials
			//p += mp;
		}
		it++;
	}

	return p;
}

/*
	Returns the potential an enemy base at (objX,objY) generates in the point (x,y).
*/
real8 PF::calcEnemyBaseCharge(int x, int y, int objX, int objY, bool inside) {
	real8 p = 0;
	if (inside) {
		p = 0;
	}
	else {
#if USE_CLOSEST_BASE_POINT == 1
		int x1 = objX - 31;
		int x2 = objX + 31;
		int y1 = objY - 31;
		int y2 = objY + 31;

		int useX = objX;
		if  (x >= x1 && x <= x2) {
			useX = x;
		}
		else {
			int dX1 = abs(x - x1);
			int dX2 = abs(x - x2);
			if (dX1 <= dX2) {
			  useX = x1;
			}
			else {
			  useX  = x2;
			}
		}
		int useY = objY;
		if  (y >= y1 && y <= y2) {
			useY = y;
		}
		else {
			int dY1 = abs(y - y1);
			int dY2 = abs(y - y2);
			if (dY1 <= dY2) {
			  useY = y1;
			}
			else {
			  useY  = y2;
			}
			
		}
		
		real8 dist = TotalPF::distance(x, y, objX, useY);
#else
		real8 dist = TotalPF::distance(x, y, objX, objY) - (real8)31; //Base width = 64
#endif
		
		if (dist <= 110) {
			p = (real8)360 / (real8)110 * dist;
		}
		else if (dist <= 112) {
			p = 360;
		}
		else {
			p = (real8)240 - (dist - 112) * 0.32;
			if (p < 0) p = 0;
		}

		p = p * TBConfig::params.OffBase_w;
	}
	return p;
}

/*
	Predicts the next position (x or y) a unit might move to.
*/
int PF::predictNext(uint4 game_tick, int p_minusOne, int p_minusTwo, int gametick_minusOne, int gametick_minusTwo) {
	//Have not found two positions yet. Cannot predict.
	if (p_minusOne == -1 || p_minusTwo == -1) {
		return -1;
	}

	real8 diff = p_minusOne - p_minusTwo;

	real8 upd_interval = gametick_minusOne - gametick_minusTwo;
	real8 cur_interval = game_tick - gametick_minusOne;
	real8 mul = cur_interval / upd_interval;
	if (mul > 1) {
		//Interval is larger than prev. Set to 1
		mul = 1;
	}

	real8 actual_diff = diff * mul;

	int res = p_minusOne + (int)round(actual_diff);

	return res;
}
