#ifndef TotalPF_H
#define TotalPF_H

#include "PF.H"
#include "GameStateModule.H"
#include "GameObj.H"
#include "Game.H"
#include "Map.H"
#include "EventTimer.H"

using namespace std;

/*
	This class holds the charge map and contain methods for updating the charges and
	returning the potential in a specific tile.
*/
class TotalPF {

	typedef GameStateModule* GameStateModulePtr;

	public:

		/*
			Constructor.
		*/
		TotalPF(GameStateModulePtr mState);

		/*
			Destructor.
		*/
		~TotalPF();

		/*
			Updates the charge a terrain tile has in a specific point.
		*/
		void updateTerrainCharge(sint4 x, sint4 y, real8 charge);

		/*
			Sets the charge a terrain tile has in a specific point.
			Previous charge in the point is discarded.
		*/
		void setTerrainCharge(sint4 x, sint4 y, real8 charge);

		
		/*
			Returns the terrain charge in a specific point.
			This method is only used in debug purpose.
		*/
		real8 getTerrainCharge(sint4 x, sint4 y);

		//Current tile width (in points) for charge map tiles
		static const int CMTILE_WIDTH;

		//The charge used for impassable terrain tiles
		static const int CHARGE_IMPASSABLE_TILES;

		//The size of the charge map
		int width;
		int half_width;
		int height;

		/*
			Returns the potential in the point (objX, objY).
			This method does not use the potential grid in PF map. Instead it calculates the correct potential
			at runtime.
		*/
		real8 getP(sint4 objX, sint4 objY, bool offensive, uint4 unitId, const Game& game, PF* pf);
		
		/*
			Returns the distance between two points (x1,y1) and (x2,y2).
		*/
		static real8 distance(sint4 x1, sint4 y1, sint4 x2, sint4 y2);

		/*
			Returns the Manhatten distance (abs(x2-x1)+abs(y2-y1)) between two points (x1,y1) and (x2,y2).
		*/
		static real8 MHdistance(sint4 x1, sint4 y1, sint4 x2, sint4 y2);

		/*
			Returns the map width in points.
		*/
		int getRealWidth();

		/*
			Returns the map height in points.
		*/
		int getRealHeight();

		/*
			Returns a point (x or y) to a tile by rounding down.
		*/
		static int posToTile(int pos);

		/*
			Returns a point (x or y) to a tile by rounding to nearest int.
		*/
		static int posToTileR(int pos);

	private:

		//Charge map point
		struct point {
			real8 terrainCharge;
		};

		//The map
		point map[128][128];

};

#endif
