#include "FCStructureItem.H"
#include "Blackboard.H"
#include "UnitAgent.H"
#include "TBConfig.H"

/*
	Adds an attacking unit for this target.
*/
void FCStructureItem::addAttacker(uint4 unitId, GameObjPtr unit) {
	AttackUnit item;
	item.unitId = unitId;
	item.unitType = Blackboard::getType(unit);
	units.push_back(item);
}

/*
	Returns true if there are enough attacking units to destroy
	this target.
*/
int FCStructureItem::canBeDestroyed() {

	//Get armor for the target type
	int armor = 0;
	if (targetUnitType == Blackboard::TYPE_TANK) {
		armor = UnitAgent::TANK_ARMOR;
	}
	if (targetUnitType == Blackboard::TYPE_MARINE) {
		armor = UnitAgent::MARINE_ARMOR;
	}
	if (targetUnitType == Blackboard::TYPE_WORKER) {
		armor = UnitAgent::WORKER_ARMOR;
	}
	if (targetUnitType == Blackboard::TYPE_BASE) {
		armor = UnitAgent::CONTROLCENTER_ARMOR;
	}
	if (targetUnitType == Blackboard::TYPE_BARRACKS) {
		armor = UnitAgent::BARRACKS_ARMOR;
	}
	if (targetUnitType == Blackboard::TYPE_FACTORY) {
		armor = UnitAgent::FACTORY_ARMOR;
	}
	
	//Get mean damage for attacking units
	//int meanDmgMarine = getValueBetween(UnitAgent::MARINE_MIN_DMG, UnitAgent::MARINE_MAX_DMG, TBConfig::params.Attack_DmgProbability) - armor;
	//int meanDmgMarine = getValueBetween(UnitAgent::MARINE_MAX_DMG, UnitAgent::MARINE_MAX_DMG, 1) - armor;
	int meanDmgMarine = getValueBetween(UnitAgent::MARINE_MIN_DMG, UnitAgent::MARINE_MIN_DMG, 0) - armor;
	int meanDmgTank = getValueBetween(UnitAgent::TANK_MIN_DMG, UnitAgent::TANK_MAX_DMG, TBConfig::params.Attack_DmgProbability) - armor;
	
	int HPcnt = HPleft;

	int attackTypeDmg = 0;
	vector<AttackUnit>::iterator it = units.begin();
	while (it != units.end()) {
		if ((*it).unitType == Blackboard::TYPE_TANK) {
			HPcnt -= meanDmgTank;
			attackTypeDmg = meanDmgTank;
		}
		if ((*it).unitType == Blackboard::TYPE_MARINE) {
			HPcnt -= meanDmgMarine;
			attackTypeDmg = meanDmgMarine;
		}
		it++;
	}

	int res = -1; //Cant be destroyed
	if (HPcnt < 0) {
		//Can be destroyed
		res = 0;
	}
	if (HPcnt <= -attackTypeDmg) {
		//Can be destroyed and there are still units left that can attack
		res = (HPcnt * -1) / attackTypeDmg;
	}

	return res;
}

/*
	Returns a value that is min + (max-min)*p%.
*/
int FCStructureItem::getValueBetween(int min, int max, real8 p) {
	real8 range = (real8)max - (real8)min;
	real8 res = (real8)min + range * p;
	return (int)res;
}

/*
	Removes a specified attacker unit from the list (if found).
*/
void FCStructureItem::removeAttacker(uint4 unitId) {
	vector<AttackUnit>::iterator it = units.begin();
	while (it != units.end()) {
		if ((*it).unitId == unitId) {
			units.erase(it);
			return;
		}
		it++;
	}
}

/*
	Returns true if this target is attacked by the specified attacker unit.
*/
bool FCStructureItem::attackedBy(uint4 unitId) {
	vector<AttackUnit>::iterator it = units.begin();
	while (it != units.end()) {
		if ((*it).unitId == unitId) {
			return true;
		}
		it++;
	}
	return false;
}

/*
	Returns the number of attacking units for this target.
*/
uint4 FCStructureItem::getNoAttackers() {
	return units.size();
}

/*
	Returns the object Id for the attacking unit at position [index] in the list.
*/
uint4 FCStructureItem::getAttackerId(uint4 index) {
	return units[index].unitId;
}

//Operator overloading used for sorting the list
bool FCStructureItem::operator==(const FCStructureItem &item) const {
	if (this->HPleft != item.HPleft) {
		return false;
	}

	//HPleft matches. Check number of attackers
	if (this->units.size() == item.units.size()) {
		return true;
	}

	return false;
}

//Operator overloading used for sorting the list
bool FCStructureItem::operator<(const FCStructureItem &item) const {
	if (this->HPleft == item.HPleft) {
		//> = prefer targets many units can reach (i.e. center of enemy unit groups)
		//< = prefer targets few units can reach (i.e. flanks of enemy unit groups or units at longer distance)
		if (TBConfig::hasAddition3()) {
			//Addition #2: Prioritize to attack targets at flanks
			return this->units.size() < item.units.size(); //< works best
		}
		else {
			return this->units.size() > item.units.size();
		}
	}
	else {
		return this->HPleft < item.HPleft;
	}
}

/*
	Dumps the data structure entry to the console window for debug purpose.
*/
void FCStructureItem::dump() {
	cout << targetId << "(" << HPleft << ") can be attacked by ";

	vector<AttackUnit>::iterator it = units.begin();
	while (it != units.end()) {
		cout << (*it).unitId << "(" << (*it).unitType << "), ";
		it++;
	}
	if (canBeDestroyed() >= 0) {
		cout << "CAN BE DESTROYED";
	}

	cout << "" << endl;
}
