
#include "PFUpdater.H"
#include "Map.H"
#include "PlayerInfo.H"
#include "Blackboard.H"
#include "TBConfig.H"
#include "Options.H"
#include "EventTimer.H"

bool is_initiated = false;

PFUpdater::PFUpdater() {
	
}

PFUpdater::~PFUpdater() {
	
}

TotalPF* PFUpdater::getTotalPF() {
	return c_map;
}

PF* PFUpdater::getPF() {
	return obj_map;
}

bool PFUpdater::isReady() {
	if (is_initiated) {
		return true;
	}

	return false;
}

bool PFUpdater::isInitiated() {
	if (!is_initiated) {
		return false;
	}
	return true;
}

void PFUpdater::initPF(GameStateModulePtr mState) {
	state = mState;

	playerId = state->get_game().get_client_player();
	enemyId = 0;
	if (playerId == 0) enemyId = 1;

	c_map = new TotalPF(state);

	obj_map = new PF();
	
	Options::get("-debug", debug);
	
	is_initiated = true;
}

void PFUpdater::updateUnitPF(const Game& game) {
	obj_map->setAllDestroyed();

	const sint4 my_id = game.get_client_player();
	const sint4 en_id = 1 - my_id;
	
	const Game::ObjCont &objs = game.get_objs(my_id);
	FORALL (objs, it) {
		GameObj* gob = (*it)->get_GameObj();
		if (gob && gob->sod.in_game && *gob->sod.x >= 0 && !gob->is_dead()) {
			std::string objType = gob->bp_name();
			if (objType == "marine") {
				uint4 unitId = game.get_cplayer_info().get_id(gob);
				obj_map->updateUnitPosition(game.tick(), gob, unitId, Blackboard::TYPE_MARINE, *gob->sod.x, *gob->sod.y, gob->get_int("hp"));
			}
			if (objType == "tank") {
				uint4 unitId = game.get_cplayer_info().get_id(gob);
				obj_map->updateUnitPosition(game.tick(), gob, unitId, Blackboard::TYPE_TANK, *gob->sod.x, *gob->sod.y, gob->get_int("hp"));
			}
			if (objType == "worker") {
				uint4 unitId = game.get_cplayer_info().get_id(gob);
				obj_map->updateUnitPosition(game.tick(), gob, unitId, Blackboard::TYPE_WORKER, *gob->sod.x, *gob->sod.y, gob->get_int("hp"));
			}
		}
	}

	const Game::ObjCont &eobjs = game.get_objs(en_id);
	FORALL (eobjs, it) {
		GameObj* gob = (*it)->get_GameObj();
		if (gob && gob->sod.in_game && *gob->sod.x >= 0 && !gob->is_dead()) {
			std::string objType = gob->bp_name();
			if (objType == "marine") {
				uint4 unitId = game.get_cplayer_info().get_id(gob);
				obj_map->updateEnemyUnitPosition(game.tick(), unitId, Blackboard::TYPE_MARINE, *gob->sod.x, *gob->sod.y, gob->get_int("hp"));
			}
			if (objType == "tank") {
				uint4 unitId = game.get_cplayer_info().get_id(gob);
				obj_map->updateEnemyUnitPosition(game.tick(), unitId, Blackboard::TYPE_TANK, *gob->sod.x, *gob->sod.y, gob->get_int("hp"));
			}
			if (objType == "worker") {
				uint4 unitId = game.get_cplayer_info().get_id(gob);
				obj_map->updateEnemyUnitPosition(game.tick(), unitId, Blackboard::TYPE_WORKER, *gob->sod.x, *gob->sod.y, gob->get_int("hp"));
			}
		}
	}

	int n = game.get_player_num();
	const Game::ObjCont &nobjs = game.get_objs(n);
	FORALL (nobjs, it) {
		GameObj* gob = (*it)->get_GameObj();
		if (gob && gob->sod.in_game && *gob->sod.x >= 0 && !gob->is_dead()) {
			std::string objType = gob->bp_name();
			if (objType == "sheep") {
				uint4 sheepId = game.get_cplayer_info().get_id(gob);
				obj_map->updateSheepPosition(game.tick(), sheepId, *gob->sod.x, *gob->sod.y);
			}
			if (objType == "mineral") {
				uint4 mineralId = game.get_cplayer_info().get_id(gob);
				obj_map->updateMineralPosition(mineralId, *gob->sod.x, *gob->sod.y);
			}
		}
	}
}

void PFUpdater::removeBase(int x, int y) {
	obj_map->removeBase(x, y);
}

void PFUpdater::initBasePF(const Game& game) {
	obj_map->clearBaseLists();
	updateBasePF(game);
}

void PFUpdater::updateBasePF(const Game& game) {
	int n = game.get_player_num();
	PlayerInfo& pi = game.get_cplayer_info();
	const sint4 cid = game.get_client_player();
	
	//Iterate through all players. The last player (n) is the server objects, but we dont
	//need to change charge for neutral server objects.
	for (int player = 0; player < n; player++) {
		const Game::ObjCont &objs = game.get_objs(player);

		bool friendly = false;
		if (player == cid) {
			friendly = true;
		}

		//Iterate through all objects for this player
		FORALL (objs, it) {
			GameObjPtr gob = (*it)->get_GameObj();
			if (gob && gob->sod.in_game && *gob->sod.x >= 0 && !gob->is_dead()) {
				//Valid object found
				uint4 objId = pi.get_id(gob);
				
				std::string objType = gob->bp_name();
				
				int type = -1;
				if (objType == "controlCenter") {
					type = Blackboard::TYPE_BASE;
				}
				else if (objType == "barracks") {
					type = Blackboard::TYPE_BARRACKS;
				}
				else if (objType == "factory") {
					type = Blackboard::TYPE_FACTORY;
				}

				if (type != -1) {
					if (friendly) {
						obj_map->addBase(objId, gob, (*gob->sod.x), (*gob->sod.y), (*gob->sod.x1), (*gob->sod.y1), (*gob->sod.x2), (*gob->sod.y2), true, type);
						int tX1 = *gob->sod.x1 / 16 - 1;
						int tX2 = *gob->sod.x2 / 16 + 1;
						int tY1 = *gob->sod.y1 / 16 - 1;
						int tY2 = *gob->sod.y2 / 16 + 1;
						
						c_map->blockAreaFromFill(tX1, tX2, tY1, tY2);
						
					}
					else {
						obj_map->addBase(objId, gob, (*gob->sod.x), (*gob->sod.y), (*gob->sod.x1), (*gob->sod.y1), (*gob->sod.x2), (*gob->sod.y2), false, type);
					}
				}
			}
		}
	}
}

