#ifndef PFUpdater_H
#define PFUpdater_H

#include "Game.H"
#include "TotalPF.H"
#include "PF.H"
#include "GameObj.H"

using namespace std;

/*
	This class updates the positions of units and game objects, adds newly explored objects and removes destroyed objects.
*/
class PFUpdater {

	typedef GameObj* GameObjPtr;
	typedef GameStateModule* GameStateModulePtr;

	public:

		/*
			Constructor.
		*/
		PFUpdater();

		/*
			Destructor.
		*/
		~PFUpdater();

		/*
			Returns true if the potential field classes are initiated and ready to use.
		*/
		bool isInitiated();

		/*
			Returns true if all pre-calculations are ready and the potential field is ready to use.
		*/
		bool isReady();

		/*
			Initiates the potential field classes.
		*/
		void initPF(GameStateModulePtr mState);

		/*
			Update positions etc. for units. Must be called each frame.
			The method also updates minerals.
		*/
		void updateUnitPF(const Game& game);

		/*
			Update positions etc. for bases and structures. Must be called when a structure is built, destroyed
			or discovered.
		*/
		void updateBasePF(const Game& game);

		/*
			Inits the base positions etc. The difference between this method and updateBasePF is that this
			cleares the bases list and remakes it.
		*/
		void initBasePF(const Game& game);

		/*
			Removes a structure at position (x,y) from the bases list. Used when structures
			are destroyed.
		*/
		void removeBase(int x, int y);
		
		/*
			Returns the potential field class.
		*/
		PF* getPF();

		/*
			Returns the total potential field class.
		*/
		TotalPF* getTotalPF();

		
		//Global Id for the player
		sint4 playerId;
		//Global Id for the enemy
		sint4 enemyId;


		

		

	private:
		
		//The PF.
		PF* obj_map;

		//The charge map
		TotalPF* c_map;

		//Current game state. Only used in the init thread.
		GameStateModulePtr state;

		//Set to true if the client is started in debug mode
		bool debug;
		

};

#endif
