#ifndef TBConfig_H
#define TBConfig_H

#include "GameObj.H"

#define DYNAMIC_AI 0
#define LIMIT_WORKERS_PER_MINERAL 0
#define USE_CLOSEST_BASE_POINT 1
#define USE_CONVEX_FILLING 1

using namespace std;

/*
	Struct containing the changeable parameters.
*/
struct Params {
	real8 OffTank_w;
	real8 DefTank_w;
	real8 Tank_w;
	real8 Base_w;
	real8 Sheep_w;
	real8 OffBase_w;
	real8 Terrain_w;

	int TotalPF_PathMinThreshold;
	int TotalPF_PathMaxThreshold;
	real8 Attack_DmgProbability;
	bool Addition1;
	bool Addition2;
	bool Addition3;
	bool Addition4;
};

/*
	This class reads application parameters from file and stores them in a struct.
	It enables the possibility to change the AI behavior with settings in a config file.
	The path to the config file is passed as parameter when starting the app.
*/
class TBConfig {
	public:

		/*
			Sets path to the config file to use.
		*/
		static void setConfigPath(string confpath);

		/*
			Sets the app params as a string instead of read from file.
		*/
		static void setConfigString(string mParamsString);

		/*
			Returns true if config is initiated, false otherwise.
		*/
		static bool isInitiated();

		/*
			Inits the config by reading a config file and set the
			parameters in the params struct.
		*/
		static void init();

		/*
			Parses a line from the config file and, if it is a parameter line,
			sets the value in the params struct.
		*/
		static void parseLine(string line);

		/*
			Splits a string using a specified delimiter. The tokens are stored in a
			results vector.
		*/
		static int split(const string& input, const string& delimiter, vector<string>& results);

		/*
			Returns true if the specified addition is active.
		*/
		static bool hasAddition1();
		static bool hasAddition2();
		static bool hasAddition3();
		static bool hasAddition4();

		/*
			Returns true if 'line' contains 'token'.
		*/
		static bool contains(string line, string token);

		/*
			Converts a string to a real value.
			Returns 0 if string is not a valid real.
		*/
		static real8 stringToReal(string str);

		/*
			Converts a string to an int value.
			Returns 0 if string is not a valid int.
		*/
		static int stringToInt(string str);

		/*
			Converts a string to a bool. Returns false
			if not a valid string.
		*/
		static bool stringToBool(string str);

		//Parameters struct
		static Params params;

		//Path to config file
		static string path;

		//Params as a string instead of file
		static string paramsString;
};

#endif
