
#include "TBEventHandler.H"
#include "GameStateModule.H"
#include "GfxModule.H"
#include "Game.H"
#include "GameChanges.H"
#include "GameTile.H"
#include "ServerObjData.H"
#include "Options.H"
#include "UnitAgent.H"
#include "WorkerAgent.H"
#include "Blackboard.H"
#include "EventTimer.H"
#include "Profiler.H"
#include "EventTimer.H"
#include "BattlePlan.H"

using namespace std;

bool TBEventHandler::handle_event(const Event& e) {
	if (e.get_who() == GameStateModule::FROM) {
    
		if (e.get_what() == GameStateModule::STOP_MSG) {
			cout << "Game stopped" << endl;
			exit(0);
		}
    
		if (e.get_what() == GameStateModule::READ_ERROR_MSG) {
			cout << "Error occured: " << state.error << endl;
			exit(1);
		}
    
		if (e.get_what() == GameStateModule::VIEW_MSG) {
      
			compute_actions();
			// Send my actions
			state.gsm->send_actions();
			
			Game &game = state.gsm->get_game();	

			// gfx and frame statistics
      
			if (state.gfxm) state.gfxm->process_changes();
			
			sint4 vf = game.get_view_frame();
			sint4 af = game.get_action_frame();

			if (state.gfxm) {
	
				// don't draw if we are far behind
				if (abs(vf-af) < 10) {
					state.gfxm->draw();
					state.just_drew = true;
				}
			}

			return true;
		}
	}
	
	return false;
}


void TBEventHandler::compute_actions() {
	EventTimer::start();

	const GameChanges &changes = state.gsm->get_changes();
	
	bool baseUpdated = false;
	bool unitDestroyed = false;

	//Current game state
	const Game &game = state.gsm->get_game();
	sint4 af = game.get_action_frame();

	if (!changes.dead_objs.empty()) {
		FORALL (state.gsm->get_changes().dead_objs, it) {
        	GameObj *gob = (*it)->get_GameObj();
			std::string objType = gob->bp_name();
			//cout << "Dead object: " << objType << endl;
			if (objType == "controlCenter" || objType == "barracks" || objType == "factory") {
				//A base has been destroyed
				baseUpdated = true;
				pfupdater.getPF()->removeBase(*gob->sod.x, *gob->sod.y);
			}
			if (objType == "tank" || objType == "marine" || objType == "worker" || objType == "mineral") {
				//A unit has been destroyed
				unitDestroyed = true;
			}
		}
	}

	if (pfupdater.isInitiated()) {
		if (!changes.new_objs.empty()) {
			FORALL (state.gsm->get_changes().new_objs, it) {
				GameObj *gob = (*it)->get_GameObj();
				std::string objType = gob->bp_name();
				//cout << "Dead object: " << objType << endl;
				//TODO: Check if the bases belong to the player
				if (objType == "controlCenter") {
					baseUpdated = true;
				}
				if (objType == "barracks") {
					baseUpdated = true;
				}
				if (objType == "factory") {
					baseUpdated = true;
				}
			}
		}
	}
	
	//This is the unique id for this player
	const sint4 cid = game.get_client_player();

	const Game::ObjCont &objs = game.get_objs(cid);
	
	bool debug;
	Options::get("-debug", debug);
	
	if (!pfupdater.isInitiated()) {
		//We need to init the pfupdater
		pfupdater.initPF(state.gsm);
		pfupdater.initBasePF(game);
		gui.setRef(pfupdater.getTotalPF(), pfupdater.getPF(), state.gsm, fieldCom.getCoordinator());

		BattlePlan* plan = chiefCom.generateBattlePlan(game);
		fieldCom.setBattlePlan(plan);
	}
	
	//if (pfupdater.isReady() && unitDestroyed) {
		//pfupdater.getPF()->cleanUp();
	//}
	if (pfupdater.isReady() && baseUpdated) {
		pfupdater.updateBasePF(game);
	}
	if (pfupdater.isReady()) {
		if (af % 10 == 0) {
			chiefCom.updateBattlePlan(&pfupdater, fieldCom.getBattlePlan());
		}
		//Everything is ready. Update the unit map each frame
		pfupdater.updateUnitPF(game);
		pfupdater.getPF()->updateExplMap(game);
		pfupdater.getPF()->updateProbabilities();
	}
	if (pfupdater.isReady() && unitDestroyed) {
		pfupdater.getPF()->cleanUp();
	}

	fieldCom.compute_actions(state.gsm, &pfupdater, &rand);

	if (Blackboard::game_type == 1) {
		if (af % 20 == 0) {
			PlayerInfo& pi = game.get_cplayer_info();
			ScriptObj *so = pi.global_obj("player");
			cout << "Resources: " << so->get_int("minerals") << endl;
		}
	}

	if (debug) {
		gui.display();
	}
}

void TBEventHandler::dumpInfo(int af) {
	ofstream fout;
	fout.open("html/tb4_data.txt", ios::app);

	sint4 seed;
  	Options::get("-seed", seed);

	fout << "seed=" << seed << " ";
	fout << "af=" << af << " ";
	fout << "ou=" << pfupdater.getPF()->getOU() << " ";
	fout << "ob=" << pfupdater.getPF()->getOB() << " ";
	fout << "eu=" << pfupdater.getPF()->getEU() << " ";
	fout << "eb=" << pfupdater.getPF()->getEB() << " ";
	fout << "avgTime=" << EventTimer::getAvgTime() << "\n";
	fout.close();
}

void TBEventHandler::dumpExplored(int af, const Game& game) {
	ofstream fout;
	fout.open("html/tb4_expl_data.txt", ios::app);

	sint4 seed;
  	Options::get("-seed", seed);
	int sec = af / 8;

	fout << "seed=" << seed << " ";
	fout << "af=" << af << " ";
	fout << "sec=" << sec << " ";
	
	int totCnt = 0;
	int unexpCnt = 0;

	const Map<GameTile> &gameMap = game.get_map();
	FORS (y, gameMap.get_height()) {
		FORS (x, gameMap.get_width()) {
			const GameTile &tile = gameMap(x,y);
			if (tile.get_typeW() == 0) { //0 = Unknown tile
				unexpCnt++;
			}
			totCnt++;
		}
	}
	double exp = (double)(totCnt - unexpCnt) / (double)totCnt * (double)100;

	fout << "exp=" << exp << "\n";

	fout.close();
}
