#ifndef TBEventHandler_H
#define TBEventHandler_H

#include "EventHandler.H"
#include "Random.H"
#include "PFUpdater.H"
#include "CommanderInField.H"
#include "CommanderInChief.H"
#include "ClientGUI.H"
#include "SDLinit.H"

class GameStateModule;
class GfxModule;


// This class gives the event handler access to all game and gfx data

struct AIState {

	GameStateModule *gsm;
	GfxModule       *gfxm; // !=0 : gfx used
	bool just_drew;  // just drew a gfx frame
	bool quit;       // true => quit game
	bool error;      // true <=> error occurred
	
	AIState(GameStateModule *gsm_, GfxModule *gfxm_) :
    gsm(gsm_), gfxm(gfxm_) {
    	just_drew = quit = error = false;
  	}

};


// This class handles server events
// In particular it generates game object actions

class TBEventHandler : public EventHandler {

public:
	
	AIState &state;

	TBEventHandler(AIState &state_, sint4 seed) : state(state_) {
		rand.srand(seed);
	}

	bool handle_event(const Event &e);

private:
	
	void compute_actions();
	Random rand; // local RNG

	CommanderInField fieldCom;
	CommanderInChief chiefCom;

	PFUpdater pfupdater;
	ClientGUI gui;
	
	void dumpInfo(int af);
	void dumpExplored(int af, const Game& game);
 
};

#endif
