#include "WorkerAgent.H"
#include "Blackboard.H"
#include "TBConfig.H"
#include "Options.H"
#include "UnitAgent.H"

const sint4 WorkerAgent::STATE_MOVE_TO_MINE = 21;
const sint4 WorkerAgent::STATE_MINE = 22;
const sint4 WorkerAgent::STATE_MOVE_TO_BASE = 23;
const sint4 WorkerAgent::STATE_BUILD_BARRACKS = 24;
const sint4 WorkerAgent::STATE_BUILD_FACTORY = 25;
const sint4 WorkerAgent::STATE_EXPLORE = 26;

void WorkerAgent::compute_actions(GameObjPtr unit, GameStateModulePtr state, PFUpdaterPtr pfupdater, Random* rand, sint4 currentState) {
	Game& game = state->get_game();
	
	uint4 objId = game.get_cplayer_info().get_id(unit);

	//Check if this agent shall do something this frame
	real8 cProb = pfupdater->getPF()->getProbability(objId);
	real8 rnd = (real8)rand->ranged_uint4(0, 100) / (real8)100;
	//cout << "cProb=" << cProb << " rnd=" << rnd;
	if (rnd > cProb) {
		//Do nothing
		//cout << " NoAction" << endl;
		return;
	}
	//cout << " Action" << endl;

	int unitX = *unit->sod.x;
	int unitY = *unit->sod.y;
	
	if (currentState == STATE_EXPLORE) {
		bool moved = canMove(unit, unitX, unitY, objId, pfupdater, game, currentState);
	}

	if (currentState == STATE_MINE) {
		bool finished = finishedMining(unit);
		if (finished) {
			pfupdater->getPF()->updateCurrentState(objId, STATE_MOVE_TO_BASE);
		}
		else {
			//Mining not finished... mine
			bool mined = canMine(unit, unitX, unitY, objId, pfupdater, game);
			if (!mined) {
				//Cant mine... move to mine
				pfupdater->getPF()->updateCurrentState(objId, STATE_MOVE_TO_MINE);
			}
		}
	}
	
	if (currentState == STATE_MOVE_TO_MINE) {
		bool moved = canMove(unit, unitX, unitY, objId, pfupdater, game, currentState);
		if (canMine(unit, unitX, unitY, objId, pfupdater, game)) {
			//Arrived
			pfupdater->getPF()->updateCurrentState(objId, STATE_MINE);
		}
	}
	
	if (currentState == STATE_MOVE_TO_BASE) {
		bool ok = returnMinerals(unit, unitX, unitY, objId, pfupdater, game);
		if (ok) {
			pfupdater->getPF()->updateCurrentState(objId, STATE_MOVE_TO_MINE);
		}
		else {
			bool moved = canMove(unit, unitX, unitY, objId, pfupdater, game, currentState);
		}
	}
	
	if (currentState == STATE_BUILD_BARRACKS) {
		if (unitX < 0) return;
		Position pos = findBuildSpot(unitX, unitY, objId, pfupdater, game);
		real8 cDist = TotalPF::distance(unitX, unitY, pos.x, pos.y);
		//cout << "[" << objId << "] I am at [" << unitX << "," << unitY << "] and [" << pos.x << "," << pos.y << "] seems to be a good build spot. dist=" << cDist << endl;
		pfupdater->getPF()->setDestination(objId, pos);

		if (Blackboard::BARRACKS_BUILT) {
			//cout << "BARRACKS BUILT" << endl;
			Blackboard::BARRACKS_BUILT = false;
			bool finished = finishedMining(unit);
			if (finished) {
				pfupdater->getPF()->updateCurrentState(objId, STATE_MOVE_TO_BASE);
			}
			else {
				pfupdater->getPF()->updateCurrentState(objId, STATE_MOVE_TO_MINE);
			}
			pfupdater->getPF()->clearDestination(objId);
			return;
		}
		else {
			if (cDist <= 10) {
				//Build base
				buildBarracks(unit);
			}
			else {
				bool moved = canMove(unit, unitX, unitY, objId, pfupdater, game, currentState);
			}
		}
	}
	
	if (currentState == STATE_BUILD_FACTORY) {
		if (unitX < 0) return;
		Position pos = findBuildSpot(unitX, unitY, objId, pfupdater, game);
		real8 cDist = TotalPF::distance(unitX, unitY, pos.x, pos.y);
		//cout << "[" << objId << "] I am at [" << unitX << "," << unitY << "] and [" << pos.x << "," << pos.y << "] seems to be a good build spot. dist=" << cDist << endl;
		pfupdater->getPF()->setDestination(objId, pos);

		if (Blackboard::FACTORY_BUILT) {
			//cout << "FACTORY BUILT" << endl;
			Blackboard::FACTORY_BUILT = false;
			bool finished = finishedMining(unit);
			if (finished) {
				pfupdater->getPF()->updateCurrentState(objId, STATE_MOVE_TO_BASE);
			}
			else {
				pfupdater->getPF()->updateCurrentState(objId, STATE_MOVE_TO_MINE);
			}
			pfupdater->getPF()->clearDestination(objId);
			return;
		}
		else {
			if (cDist <= 10) {
				//Build factory
				buildFactory(unit);
			}
			else {
				bool moved = canMove(unit, unitX, unitY, objId, pfupdater, game, currentState);
			}
		}
	}
}

Position WorkerAgent::findBuildSpot(int unitX, int unitY, uint4 objId, PFUpdaterPtr pfupdater, const Game& game) {
	
	Position spot;
	spot.x = 0;
	spot.y = 0;

	//Check if build spot exists
	Position tmp = pfupdater->getPF()->getDestination(objId);
	if (tmp.x > 0 || tmp.y > 0) {
		//Destination exists
		return tmp;
	}

	int cDiff = 4;
	real8 bestP = -1;
	real8 p1 = 0;
	
	while(cDiff < 200) { //200
		for (int mX = unitX - cDiff; mX < unitX + cDiff; mX += 4) {
			p1 = pfupdater->getTotalPF()->getBuildBaseP(mX, unitY - cDiff, objId, game, pfupdater->getPF(), false);
			if (p1 > bestP) {
				spot.x = mX;
				spot.y = unitY - cDiff;
				bestP = p1;
			}

			p1 = pfupdater->getTotalPF()->getBuildBaseP(mX, unitY + cDiff, objId, game, pfupdater->getPF(), false);
			if (p1 > bestP) {
				spot.x = mX;
				spot.y = unitY + cDiff;
				bestP = p1;
			}
		}

		for (int mY = unitY - cDiff; mY < unitY + cDiff; mY += 4) {
			p1 = pfupdater->getTotalPF()->getBuildBaseP(unitX - cDiff, mY, objId, game, pfupdater->getPF(), false);
			if (p1 > bestP) {
				spot.x = unitX - cDiff;
				spot.y = mY;
				bestP = p1;
			}

			p1 = pfupdater->getTotalPF()->getBuildBaseP(unitX + cDiff, mY, objId, game, pfupdater->getPF(), false);
			if (p1 > bestP) {
				spot.x = unitX + cDiff;
				spot.y = mY;
				bestP = p1;
			}
		}

		cDiff += 4;
	}

	cout << "[" << objId << "] Found buildspot at (" << spot.x << "," << spot.y << ")" << endl;

	pfupdater->getPF()->setDestination(objId, spot);

	return spot;
}

bool WorkerAgent::returnMinerals(GameObjPtr unit, int unitX, int unitY, uint4 objId, PFUpdaterPtr pfupdater, const Game& game) {
	vector<BasePos>* bases = pfupdater->getPF()->getBasesList();

	vector<BasePos>::iterator it = bases->begin();
	while (it != bases->end()) {
		if (it->isPositionInside(unitX, unitY, 1)) {
			Vector<sint4> params;
			params.push_back(it->objId);
			unit->set_action("return_resources", params);
			return true;
		}
		it++;
	}

	return false;
}

bool WorkerAgent::finishedMining(GameObjPtr unit) {
	int currentMinerals = (int)unit->get_int("minerals");
	if (currentMinerals >= 10) {
		return true;
	}
	return false;
}

bool WorkerAgent::canMine(GameObjPtr unit, int unitX, int unitY, uint4 objId, PFUpdaterPtr pfupdater, const Game& game) {
	vector<MineralPos>* minerals = pfupdater->getPF()->getMineralsList();

	vector<MineralPos>::iterator it = minerals->begin();
	while (it != minerals->end()) {
		if (!it->destroyed) {
			real8 dist = TotalPF::distance(unitX, unitY, it->x, it->y);
			if (dist <= 10) {
				Vector<sint4> params;
				params.push_back(it->objId);
				ScriptObj *pickaxe = unit->component("pickaxe");
				pickaxe->set_action("mine", params);
				return true;
			}
		}
		it++;
	}
	
	return false;
}

/*
	Checks if there are better positions nearby to move to.
	Returns true if unit moves, false if not.
*/
bool WorkerAgent::canMove(GameObjPtr unit, int unitX, int unitY, uint4 objId, PFUpdaterPtr pfupdater, const Game& game, sint4 currentState) {
	//If no targets are within range, check if there are better positions to move to
	TotalPFPtr pf = pfupdater->getTotalPF();

	int maxSpeed = *unit->sod.max_speed;
	
	//Calculate which tiles to check
	int x1 = unitX - maxSpeed;
	int x2 = unitX + maxSpeed;
	int y1 = unitY - maxSpeed;
	int y2 = unitY + maxSpeed;

	//Iterate through the tiles to get the highest (=best) charge
	//Start with the charge for current tile
	real8 bestCharge = pf->getWorkerP(unitX, unitY, currentState, objId, Blackboard::getType(unit), game, pfupdater->getPF(), true);
	
	int bestX = unitX;
	int bestY = unitY;
	
	//Get the surrounding tile with highest charge
	for (int cX = x1; cX <= x2; cX++) {
		for (int cY = y1; cY <= y2; cY++) {
			real8 penalty = 1;
#if USE_CLOSEST_BASE_POINT == 1
			if (cX != unitX && cY != unitY) {
				//Give a small penalty for moving diagonally
				penalty = 0.9999;
			}
#endif

			if (cX != unitX || cY != unitY) {
				if (UnitAgent::canDoMove(objId, unitX, unitY, cX, cY, Blackboard::TYPE_WORKER, pfupdater, game)) {
					real8 cCharge = pf->getWorkerP(cX, cY, currentState, objId, Blackboard::getType(unit), game, pfupdater->getPF(), true) * penalty;
					if (cCharge > bestCharge) {
						bestCharge = cCharge;
						bestX = cX;
						bestY = cY;
					}
				}
			}
		}
	}
	
	//If we've found a surrounding tile with higher charge than current tile, move there
	if (bestX != unitX || bestY != unitY) {
		int useSpeed = *unit->sod.max_speed;
		if (pf->distance(unitX, unitY, bestX, bestY) < useSpeed) {
			useSpeed = pf->distance(unitX, unitY, bestX, bestY);
		}
		pfupdater->getPF()->setNextUnitPosition(objId, bestX, bestY);

		Vector<sint4> params;
		params.push_back(bestX);
		params.push_back(bestY);
		params.push_back(useSpeed);

		unit->set_action("move", params);

		//Unit has moved
		return true;
	}

	//Unit has not moved
	return false;
}

void WorkerAgent::buildBarracks(GameObjPtr unit) {
	int x = *unit->sod.x;
	int y = *unit->sod.y;

	Vector<sint4> args;
	args.push_back(x);
	args.push_back(y);
	unit->set_action("build_barracks", args);
}

void WorkerAgent::buildFactory(GameObjPtr unit) {
	int x = *unit->sod.x;
	int y = *unit->sod.y;

	Vector<sint4> args;
	args.push_back(x);
	args.push_back(y);
	unit->set_action("build_factory", args);
}

void WorkerAgent::buildControlCenter(GameObjPtr unit) {
	int x = *unit->sod.x;
	int y = *unit->sod.y;

	Vector<sint4> args;
	args.push_back(x);
	args.push_back(y);
	unit->set_action("build_controlCenter", args);
}

