#ifndef WorkerAgent_H
#define WorkerAgent_H

#include "TotalPF.H"
#include "PF.H"
#include "PFUpdater.H"
#include "GameObj.H"
#include "Game.H"
#include "GameStateModule.H"

/*
	This class handles all actions available for own worker units
*/
class WorkerAgent {
	
	typedef TotalPF* TotalPFPtr;
	typedef GameObj* GameObjPtr;
	typedef PFUpdater* PFUpdaterPtr;
	typedef GameStateModule* GameStateModulePtr;

	public:

		/*
			Main "execute action" method for each worker. Call this method each frame.
		*/
		static void compute_actions(GameObjPtr unit, GameStateModulePtr state, PFUpdaterPtr pfupdater, Random* rand, sint4 currentState);

		/*
			Returns carried minerals to a controlCenter. The worker must be within range for this method to work.
		*/
		static bool returnMinerals(GameObjPtr unit, int unitX, int unitY, uint4 objId, PFUpdaterPtr pfupdater, const Game& game);

		/*
			Returns true if this worker has finished mining, i.e. he cant carry more minerals.
		*/
		static bool finishedMining(GameObjPtr unit);

		/*
			Mines from a mineral patch, if possible. Returns true if the worker can mine, false if not.
		*/
		static bool canMine(GameObjPtr unit, int unitX, int unitY, uint4 objId, PFUpdaterPtr pfupdater, const Game& game);

		/*
			Checks if there are better positions nearby to move to.
			Returns true if unit moves, false if not.
		*/
		static bool canMove(GameObjPtr unit, int unitX, int unitY, uint4 objId, PFUpdaterPtr pfupdater, const Game& game, sint4 currentState);

		/*
			Finds a good spot to build a new structure at. If the method returns (0,0) no good buildspot was found.
		*/
		static Position findBuildSpot(int unitX, int unitY, uint4 objId, PFUpdaterPtr pfupdater, const Game& game);

		/*
			Build barracks.
		*/
		static void buildBarracks(GameObjPtr unit);

		/*
			Build a factory.
		*/
		static void buildFactory(GameObjPtr unit);

		/*
			Build a base/controlCenter.
		*/
		static void buildControlCenter(GameObjPtr unit);

		//Worker FSM state variables
		static const sint4 STATE_MOVE_TO_MINE;
		static const sint4 STATE_MINE;
		static const sint4 STATE_MOVE_TO_BASE;
		static const sint4 STATE_BUILD_BARRACKS;
		static const sint4 STATE_BUILD_FACTORY;
		static const sint4 STATE_EXPLORE;
};

#endif
